/* Serial TX portion taken from https://www.avrfreaks.net/forum/software-serial-uart?page=all 
   Post dated May 1, 2013 by Postman 
   (referring to an @clawson as the originator of the code)
   
   Serial RX Written by dekohn based on Software UART flowchart 
   found in https://www.renesas.com/eu/en/doc/DocumentServer/021/U13218EU1V0AN00.pdf 
   on page 58.
   
   AVR Pin	Arduino Pin		Function				EPS8266
   PB0		8				TX						RX
   PD2		2				RX (INT0)				TX
							3.3v					3.3v
							GND						GND
							3.3v					EN
							
   
 */


#include <atmel_start.h>
#include <stdio.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>


#define BAUD 9600
#define STOP_BITS 1

#define SOFT_TX_BIT 0
#define SOFT_TX_DDR DDRB
#define SOFT_TX_PORT PORTB

#define SOFT_RX_BIT 2
#define SOFT_RX_DDR DDRD
#define SOFT_RX_PORT PIND

#define MICROSECONDS_OVERHEAD_ADJUST 0

#define MICROSECONDS_PER_BIT ((1000000ul / BAUD) - MICROSECONDS_OVERHEAD_ADJUST)


volatile uint8_t rx_char;
volatile uint8_t rx_flag=0;
volatile char x[1024];
volatile int c=0;

void serial_configure();
int usart_putchar (char);
void usart_putstr (char *);
void usart_response ();


int main(void)
{

	char string[255];
	
	/* Initializes MCU, drivers and middleware */
	atmel_start_init();

	PRR=0;
	
	serial_configure();
	
	_delay_ms(1000);
	
	printf("TEST RUNNING\n\r");
	

	/* Replace with your application code */
	while (1) {
		
		c=0;
		
		// Send AT 
		sprintf(string,"AT\r\n");
		usart_putstr(string);
		
		//Get and print response
		usart_response();
		
	}
}


// Software serial receive
// INT0 looks for the start bit (Falling Edge of pin) 
// then each bit is timed using standard timing. 
ISR(INT0_vect)
{

	_delay_us(MICROSECONDS_PER_BIT*1.5);				// time delay from start bit falling edge to center of D0

	
	uint8_t  bit_mask;
	uint8_t  temp;
	
	temp=0;
	for (bit_mask=0x01; bit_mask; bit_mask<<=1) {		// if bit a 1, write into correct bit (not nessisary for zeros)
		if((SOFT_RX_PORT&(1<<SOFT_RX_BIT))) {			
			temp |= bit_mask;
		}
		_delay_us(MICROSECONDS_PER_BIT);				// wait for time of one bit (center of next data bit)
	}
	
//	_delay_us(MICROSECONDS_PER_BIT*2);					// was for stop bits, but found to be unnessisary 


	x[c]=temp;
	c++;
	EIFR=0x01;											// clear int flag (since any falling edge of the pin will 
														// cause the interrupt flag to be set)
}



void usart_response ()
{
			_delay_ms(1000);
			x[c]='\0';
			
			printf("%s**\t%i\n\r",x,c);
			
			c=0;
}

void serial_configure() {

	SOFT_TX_PORT |= (1<<SOFT_TX_BIT);
	SOFT_TX_DDR |= (1<<SOFT_TX_BIT);
//	fdev_setup_stream (&usartout, usart_putchar, NULL, _FDEV_SETUP_WRITE);
//	stdout = &usartout;

    SOFT_RX_DDR &= ~(1<<SOFT_RX_BIT);
//	was not used: SOFT_RX_PORT &= ~(1<<SOFT_RX_BIT); 

    EIMSK = 1<<INT0;
 	EICRA = (1<<ISC01)|(0<<ISC00);	
	
	sei();

}


int usart_putchar (char c) {

	cli();										//prevent interrupt for incoming characters
	uint8_t  bit_mask;

	// start bit
	SOFT_TX_PORT &= ~(1<<SOFT_TX_BIT);
	_delay_us(MICROSECONDS_PER_BIT);

	// data bits
	for (bit_mask=0x01; bit_mask; bit_mask<<=1) {
		if (c & bit_mask) {
			SOFT_TX_PORT |= (1<<SOFT_TX_BIT);
		}
		else {
			SOFT_TX_PORT &= ~(1<<SOFT_TX_BIT);
		}
		_delay_us(MICROSECONDS_PER_BIT);
	}

	// stop bit(s)
	SOFT_TX_PORT |= (1<<SOFT_TX_BIT);
	_delay_us(MICROSECONDS_PER_BIT);

	EIFR=0x01;															//clear int flag
	
	sei();																//turn back on interrupt
	return c;

}

void usart_putstr(char send_buff[255])
{
	for (int x=0;x<strlen(send_buff);x++)
	{
		usart_putchar(send_buff[x]);
		_delay_us(100);
	}
}

